export const isValue = (val: any) => {
  if (val === null || val === undefined || val === '') {
    return false
  }
  return true
}

export const structureParams = (array: Array<any>, attribute: string) => {
  const endArray: any[] = []
  array.forEach(v => {
    endArray.push(v[attribute])
  })
  return endArray
}

export const replaceParams = (array: Array<any>, reArray: Array<any>, attribute: string, reAttribute: string) => {
  const endArray: any[] = []
  const endAllArray: any[] = []
  array.forEach(v => {
    reArray.forEach(rv => {
      if (v === rv[attribute]) {
        endArray.push(rv[reAttribute])
        endAllArray.push(rv)
      }
    })
  })
  return {
    replace: endArray,
    all: endAllArray
  }
}

export const copyObject = (ob: Object) => {
  return JSON.parse(JSON.stringify(ob))
}

export const arrayToMap = (array: Array<any>, key: any) => {
  const map = new Map()
  array.forEach((v: any) => {
    map.set(v[key], v)
  })
  return map
}

/**
 * 通过某个字段在一个多级数组中查找数据
 * @param data 目标数组，不能包含函数
 * @param current 目标数据
 * @param key 查找的字段
 * @param children 子集集合字段
 */
export const findInListData = (data: Array<any>, current:any, key = "id", children = 'children') => {

  for(let item of data){

    if(item[key] && JSON.parse(JSON.stringify(item[key])) == JSON.parse(JSON.stringify(current))) return item

    if(!!item[children] && Array.isArray(item[children]) && item[children].length > 0){

      const findChildData = findInListData(item[children], current, key, children)

      if(findChildData) return findChildData

    }

  }

  return null
}

export const formatGetParam = (params: Object) => {
  let paramUrl = ''
  Object.keys(params).forEach((v, i) => {
    paramUrl += i === 0 ? `${v}=${encodeURIComponent(params[v])}` : `&${v}=${encodeURIComponent(params[v])}`
  })
  return paramUrl
}

export const formatTableHeadFilters = (arr, text = 'dictLabel', value = 'dictValue') => {
  return arr.map(v => {
    v.value = v[value]
    v.text = v[text]
    return v
  })
}

export const YMDHms = (date) => {
  const _date = new Date(date)
  const Y = `${_date.getFullYear()}`;
  const M = `${_date.getMonth() + 1 < 10 ? `0${_date.getMonth() + 1}` : _date.getMonth() + 1}`;
  const D = `${_date.getDate() + 1 < 10 ? `0${_date.getDate()}` : _date.getDate()}`;
  const H = `${_date.getHours() < 10 ? `0${_date.getHours()}` : _date.getHours()}`;
  const m = `${_date.getMinutes() < 10 ? `0${_date.getMinutes()}` : _date.getMinutes()}`;
  const s = _date.getSeconds() < 10 ? `0${_date.getSeconds()}` : _date.getSeconds();
  return `${Y}-${M}-${D} ${H}:${m}:${s}`;
}

export const YMD = (date) => {
  const _date = new Date(date)
  const Y = `${_date.getFullYear()}`;
  const M = `${_date.getMonth() + 1 < 10 ? `0${_date.getMonth() + 1}` : _date.getMonth() + 1}`;
  const D = `${_date.getDate() + 1 < 10 ? `0${_date.getDate()}` : _date.getDate()}`;
  return `${Y}-${M}-${D}`;
}

//防抖
export const debounce = function (cb, ms = 0) {
  let timer = null
  return function () {
    if (timer) clearTimeout(timer)
    timer = setTimeout(() => {
      cb.apply(this, arguments)
      timer = null
    }, ms)
  }
}

export const comTime = (time) => {
  const sAll = time
  const d = Math.floor(sAll / (1000 * 60 * 60 * 24))
  const h = Math.floor((sAll - d * (1000 * 60 * 60 * 24)) / (1000 * 60 * 60))
  const m = Math.floor((sAll - d * (1000 * 60 * 60 * 24) - h * (1000 * 60 * 60)) / (1000 * 60))
  const s = Math.floor((sAll - d * (1000 * 60 * 60 * 24) - h * (1000 * 60 * 60) - m * (1000 * 60)) / 1000)
  return{
    d, h, m ,s
  }
}

export const comTimeByArea = (start, end) => {
  const sAll = new Date(end).getTime() - new Date(start).getTime()
  const d = Math.floor(sAll / (1000 * 60 * 60 * 24))
  const h = Math.floor((sAll - d * (1000 * 60 * 60 * 24)) / (1000 * 60 * 60))
  const m = Math.floor((sAll - d * (1000 * 60 * 60 * 24) - h * (1000 * 60 * 60)) / (1000 * 60))
  const s = Math.floor((sAll - d * (1000 * 60 * 60 * 24) - h * (1000 * 60 * 60) - m * (1000 * 60)) / 1000)
  return{
    d, h, m ,s
  }
}

export const deepAssign = (...obj) => {
  const result = Object.assign({}, ...obj)
  for (let item of obj) {
    for (let [idx, val] of Object.entries(item)) {
      if (val instanceof Array) {
        result[idx] = val
      } else if (val instanceof Object) {
        result[idx] = deepAssign(result[idx], val)
      }
    }
  }
  return result
}

export const copy = (value) => {
  const str = document.createElement('input')
  str.setAttribute('value', value)
  document.body.appendChild(str)
  str.select()
  document.execCommand('copy')
  document.body.removeChild(str)
  console.log(value)
}

/**
 *
 * @param precision 精度  1、0.1 、0.01……
 * @param colorArr
 * [
 *    [20.1, '#111111'],
 *    [20.3, '#dddddd'],
 *    [20.7, '#eeeeee'],
 * ]
 * @return colorMap
 * new Map([
 *    [20.1, '#111111']
 *    ……
 *    [20.3, '#dddddd']
 *    ……
 *    [20.7, '#eeeeee']
 * ])
 */
export const getGradientColorArray = (precision, colorArr) => {
  // 将hex表示方式转换为rgb表示方式(这里返回rgb数组模式)
  const colorRgb = (sColor) => {
    const reg = /^#([0-9a-fA-f]{3}|[0-9a-fA-f]{6})$/;
    let _sColor = sColor.toLowerCase();
    if (_sColor && reg.test(_sColor)) {
      if (_sColor.length === 4) {
        let sColorNew = "#";
        for (let i = 1; i < 4; i += 1) {
          sColorNew += _sColor.slice(i, i + 1).concat(_sColor.slice(i, i + 1));
        }
        _sColor = sColorNew;
      }
      //处理六位的颜色值
      const sColorChange = [];
      for (let i = 1; i < 7; i += 2) {
        sColorChange.push(parseInt("0x" + _sColor.slice(i, i + 2)));
      }
      return sColorChange;
    } else {
      return _sColor;
    }
  };
  // 将rgb表示方式转换为hex表示方式
  const colorHex = (rgb) => {
    const _this = rgb;
    const reg = /^#([0-9a-fA-f]{3}|[0-9a-fA-f]{6})$/;
    if (/^(rgb|RGB)/.test(_this)) {
      const aColor = _this.replace(/(?:(|)|rgb|RGB)*/g, "").split(",");
      let strHex = "#";
      for (let i = 0; i < aColor.length; i++) {
        let hex = Number(aColor[i]).toString(16);
        hex = hex < 10 ? 0 + '' + hex : hex;// 保证每个rgb的值为2位
        if (hex === "0") {
          hex += hex;
        }
        strHex += hex;
      }
      if (strHex.length !== 7) {
        strHex = _this;
      }
      return strHex;
    } else if (reg.test(_this)) {
      const aNum = _this.replace(/#/, "").split("");
      if (aNum.length === 6) {
        return _this;
      } else if (aNum.length === 3) {
        let numHex = "#";
        for (let i = 0; i < aNum.length; i += 1) {
          numHex += (aNum[i] + aNum[i]);
        }
        return numHex;
      }
    } else {
      return _this;
    }
  }
  const rgb2hex = (sRGB) => {
    const reg = /^(RGB|rgb)\((\d+),\s*(\d+),\s*(\d+)\)$/
    if (!reg.test(sRGB)) return sRGB
    const rgbArr = sRGB.match(/\d+/g)
    const resultRgbArr = rgbArr.map(v => {
      if (+v > 16) return (+v).toString(16)
      return '0' + (+v).toString(16)
    })
    return '#' + resultRgbArr.join('')
  }
  const gradientColor = (startColor, endColor, step) => {
    const startRGB = colorRgb(startColor);//转换为rgb数组模式
    const startR = startRGB[0];
    const startG = startRGB[1];
    const startB = startRGB[2];
    const endRGB = colorRgb(endColor);
    const endR = endRGB[0];
    const endG = endRGB[1];
    const endB = endRGB[2];
    const sR = (endR - startR) / step;//总差值
    const sG = (endG - startG) / step;
    const sB = (endB - startB) / step;
    const colorArr = [];
    for (let i = 0; i <= step; i++) {
      //计算每一步的hex值
      const hex = colorHex('rgb(' + parseInt((sR * i + startR)) + ',' + parseInt((sG * i + startG)) + ',' + parseInt((sB * i + startB)) + ')');
      colorArr.push(rgb2hex(hex));
    }
    return colorArr;
  }
  const colorMap = new Map()
  colorArr.forEach((v, i) => {
    if (i < colorArr.length - 1) {
      const _arr = gradientColor(v[1], colorArr[i + 1][1], (Number(colorArr[i + 1][0]) - Number(v[0])) / precision)
      _arr.forEach((cV, cI) => {
        colorMap.set((Number(v[0]) + cI * precision).toFixed(String(precision).split('').filter(p => p === '0').length), cV)
      })
    } else {
      colorMap.set(Number(v[0]).toFixed(String(precision).split('').filter(p => p === '0').length), v[1])
    }
  })
  return colorMap
}
